@@ -4,7 +4,7 @@
 #
 
 package Parallel::WorkUnit;
-$Parallel::WorkUnit::VERSION = '1.114'; # TRIAL
+$Parallel::WorkUnit::VERSION = '1.115'; # TRIAL
 use v5.8;
 
 # ABSTRACT: Provide multi-paradigm forking with ability to pass back data
@@ -234,6 +234,12 @@ sub async {
 
     } else {
         # We are in the child process
+        if ($use_anyevent_pipe) {
+            $pipe = $pipe->[1];
+        } else {
+            $pipe->writer();
+        }
+        $pipe->autoflush(1);
 
         return $self->_child( $sub, $pipe, undef );
     }
@@ -249,14 +255,6 @@ sub _child {
         $wu->_clear_all( $wu == $self );
     }
 
-    # We are in the child process
-    if ($use_anyevent_pipe) {
-        $pipe = $pipe->[1];
-    } else {
-        $pipe->writer();
-    }
-    $pipe->autoflush(1);
-
     try {
         my $result = $sub->();
         $self->_send_result( $pipe, $result, $pid );
@@ -346,6 +344,7 @@ sub _waitone {
     my ($self) = @_;
 
     my $sp = $self->_subprocs();
+    weaken $sp; # To avoid some Windows warnings
     if ( !keys(%$sp) ) { return; }
 
     if ($use_thread_queue) {
@@ -617,9 +616,6 @@ sub _set_anyevent {
 
         if ( defined( $self->_subprocs() ) ) {
             foreach my $pid ( keys %{ $self->_subprocs() } ) {
-                my $proc = $self->_subprocs()->{$pid};
-                undef $proc;
-
                 $self->_add_anyevent_watcher($pid);
             }
         }
@@ -681,10 +677,15 @@ sub _clear_all {
     $self->_last_error(undef);
     $self->_ordered_count(0);
     $self->_ordered_responses( [] );
-    $self->_subprocs( {} );
     $self->_count(1);
     $self->_queued_children( [] );
 
+    do {
+        # Don't warn on AnyEvent in child threads being DEMOLISHed
+        local $SIG{__WARN__} = sub { };
+        $self->_subprocs( {} );
+    };
+
     if ( defined( $self->_queue() ) ) {
         if ($keep_queue) {
             $self->_child_queue( $self->_queue() );
@@ -714,11 +715,6 @@ sub _codelike {
 sub DEMOLISH {
     my $self = shift;
 
-    # If we use threads, just don't print this warning because it is
-    # going to happen on parent and child threads both. We want
-    # different process IDs for this warning.
-    if ($use_threads) { return; }
-
     if ( scalar( keys %{ $self->_subprocs } ) ) {
         warn "Warning: Subprocesses running when Parallel::WorkUnit object destroyed\n";
     }
@@ -742,7 +738,7 @@ Parallel::WorkUnit - Provide multi-paradigm forking with ability to pass back da
 
 =head1 VERSION
 
-version 1.114
+version 1.115
 
 =head1 SYNOPSIS
 
